/** RtspSnifferSession.h
===========================================================================
MODULE          : RtspSnifferSession

DESCRIPTIONS    : This class is used to detect media information given an rtsp URL. 
				  This class uses the liveMedia library for the RTSP communication with the RTSP server
				  It stores media subsessions in a vector which are then used to create and configure the 
				  DirectShow output pins with the correct media parameters

REVISION HISTORY: Version 1.0

COPYRIGHT       : (c)CSIR, Meraka Institute 2008 all rights reserved

RESTRICTIONS	: The information/data/code contained within this file is 
				  the property of CSIR, Meraka Institute and has been 
				  classified as PUBLIC.
===========================================================================
*/
#pragma once

// STL
#include <map>
#include <string>
#include <vector>

// LiveMedia
#include <liveMedia.hh>

class RtspSnifferSession
{
public:

	/// Constructor
	RtspSnifferSession();

	/// Constructor
	~RtspSnifferSession();
	/// This method returns the media type information necessary to construct the filter's output pins
	/// @param[in] sUrl The url of the media stream
	/// @param[out] A the vector that is going to receive the sniffed information
	/// @param[in] The RTSP client request timeout
	bool getMediaSessionInformation(const std::string sUrl, int nTimeOut = -1);

	std::vector<MediaSubsession*> getMediaSubsessions()
	{
		return m_vSubsessions;
	}

	std::string getLastError()
	{
		return m_sLastError;
	}

	/// Shutdown RTSP session cleanly
	void shutdown();

	/// Bye Handler
	static void subsessionByeHandler(void* clientData) 
	{
		RtspSnifferSession* pSnifferSession = (RtspSnifferSession*) clientData;
		pSnifferSession->shutdown();
	}

private:

	bool playStreamsIfNecessary(UsageEnvironment* pEnv, MediaSession* pSession);

	bool createReceivers(MediaSession* pSession);
	/// This method creates a mapping between the liveMedia media subsession which contains all the media information
	/// And our rtvc::MediaSubsession, which stores all data required to create the output pins
	/// We could just have used the liveMedia class and created our output pins using that but this method seperates the livemedia 
	/// Code from our own and makes our Outputpin implementation independent of the liveMedia library
	//bool mapLiveMediaToRtvc(MediaSession* pSession);

	/// Currently only H263 streams need to be played back so that we can extract the width and the height from the raw data stream
	bool needToPlayStreams(MediaSession* pSubsession);
	
	/// Format sniffer for H263
	//H263FormatSniffer* m_pFormatSniffer;

	/// Rtsp client
	RTSPClient* m_pRtspClient;

	/// MediaSession
	MediaSession* m_pSession;

	/// TCP
	bool m_bStreamUsingTCP;

	/// LiveMediaEventLoop watch variable
	char m_watchVariable;

	// Stores last error
	std::string m_sLastError;

	// Store media subsessions
	std::vector<MediaSubsession*> m_vSubsessions;
};