/** RtspSourceFilter.h
===========================================================================
MODULE          : RtspSourceFilter.h

DESCRIPTIONS    : DirectShow RTSP Source filter

REVISION HISTORY: Version 1.0

COPYRIGHT       : (c)CSIR, Meraka Institute 2008 all rights reserved

RESTRICTIONS	: The information/data/code contained within this file is 
				  the property of CSIR, Meraka Institute and has been 
				  classified as PUBLIC.
===========================================================================
*/
#pragma once

// DirectShow includes
#pragma warning(push)     // disable for this header only
#pragma warning(disable:4312) 
// DirectShow
#include <Streams.h>
#pragma warning(pop)      // restore original warning level

// STL includes
#include <string>
#include <vector>

// RTVC includes
#include "RtpPacketManager.h"
#include <DirectShow/CStatusInterface.h>
#include <DirectShow/CSettingsInterface.h>

// Forward
class RtspSourceOutputPin;
class RtspDataSession;
class MediaSubsession;

// Filter name strings
#define g_wszFilterName    L"Meraka RTVC RTSP Source Filter(Audio)"

#define STREAM_USING_TCP	"streamusingtcp"

// {CF5878AC-78F0-4544-83B9-A940E20D9690}
static const GUID CLSID_RTVC_RtspAudioSourceFilter = 
{ 0xcf5878ac, 0x78f0, 0x4544, { 0x83, 0xb9, 0xa9, 0x40, 0xe2, 0xd, 0x96, 0x90 } };


class RtspSourceFilter :	public CSource,				/* Source Filter */
							public CSettingsInterface,	/* Rtvc Settings Interface */
							public CStatusInterface,	/* Rtvc Status Interface */
							public IFileSourceFilter,	/* To facilitate loading of URL */
							public IAMFilterMiscFlags	/* For Live Source purposes */
{
	///this needs to be declared for the extra interface (adds the COM AddRef, etc methods)
	DECLARE_IUNKNOWN;

	/// Give the pin friend access
	friend class RtspSourceOutputPin;

public:
	/// DLL factory method
	static CUnknown *WINAPI CreateInstance(IUnknown* pUnk, HRESULT* phr);

	/// override this to publicize our interfaces
	STDMETHODIMP NonDelegatingQueryInterface(REFIID riid, void **ppv);
	
	/// From CSettingsInterface
	virtual void initParameters()
	{
		addParameter(STREAM_USING_TCP, &m_bStreamUsingTCP, true);
	}

	/// From IFileSourceFilter
	STDMETHODIMP Load(LPCOLESTR lpwszFileName, const AM_MEDIA_TYPE *pmt);
	/// From IFileSourceFilter
	STDMETHODIMP GetCurFile(LPOLESTR * ppszFileName, AM_MEDIA_TYPE *pmt);

	/// From IAMFilterMiscFlags
	virtual ULONG STDMETHODCALLTYPE GetMiscFlags()
	{
		return AM_FILTER_MISC_FLAGS_IS_SOURCE;
	}

	/// CBase Filter methods - Overridden since we don't just have one input and output pin as the standard transform filter does
	virtual int GetPinCount();
	/// Override this method: the pins get created in this method of the transform filter
	virtual CBasePin * GetPin(int n);
	/// Method needed to connect pins based on their names
	virtual STDMETHODIMP FindPin(LPCWSTR Id, IPin **ppPin);

	/// From CSource
	STDMETHODIMP Stop();
	/// From CSource
	STDMETHODIMP Pause();
	
	/// From CBaseFilter TO PREVENT THE VIDEO RENDERER FROM BLOCKING: READ http://msdn2.microsoft.com/en-us/library/ms783675(VS.85).aspx
	STDMETHODIMP GetState( DWORD dwMilliSecsTimeout, FILTER_STATE *State	);

	/// This method starts an RTSP session in a new thread provided one hasn't been started already. In that case this method does nothing
	void StartRtspServerThreadIfNotStarted();
	/// Starts an RTSP session. This is just a helper method which is called from the thread spawned by StartRtspServerThreadIfNotStarted
	void StartRtspSession();

	/// Creates an output pin based on the passed in MediaSubsession and adds it to the vector of output pins
	/// @param[in] pSubsession a liveMedia MediaSubsession that has been obtained using RTSP and contains information such as media type, subtype- etc.
	/// @param[out] pHr Set this value to S_OK before passing pHr into this method. If the method fails, the error code will be returned in this out parameter.
	/// @remark Currently creates pins for 8/16 bit PCM audio and the RTVC H263 video media formats
	void createOutputPin(MediaSubsession *pSubsession, HRESULT* phr);

private:

	///Private Constructor
	RtspSourceFilter(IUnknown* pUnk, HRESULT* phr);
	virtual ~RtspSourceFilter(void);

	/// Notifies the filter of the start time offset.
	/// The filter will set this offset on all output pins.
	void notifyFilterAboutOffset(double dOffset);

	/// vector of output pins
	std::vector<RtspSourceOutputPin*> m_vOutputPins;

	/// Url
	std::string m_sUrl;

	CCritSec m_stateLock;

	/// Stores whether this filter should connect using TCP or UDP
	bool m_bStreamUsingTCP;

	/// Stores streaming state
	bool m_bStreaming;

	/// Handle to let the filter know that the RTSP thread has finished
	/// The Stop method waits for a signal that the liveMedia event loop has ended before it proceeds
	HANDLE m_hLiveMediaStopEvent;

	/// Stream time offset
	REFERENCE_TIME m_tStreamTimeOffset;
	double m_dStreamTimeOffset;

	// RTP Packet Manager
	RtpPacketManager m_rtpPacketManager;
	
	/// Rtsp Session
	RtspDataSession* m_pRtspDataSession;
};
