/** RtspSourceOutputPin.h
===========================================================================
MODULE          : RtspSourceOutputPin

DESCRIPTIONS    : Output pin for RtspSourceFilter
				  This pin currently caters for PCM output media types

REVISION HISTORY: Version 1.0

COPYRIGHT       : (c)CSIR, Meraka Institute 2008 all rights reserved

RESTRICTIONS	: The information/data/code contained within this file is 
				  the property of CSIR, Meraka Institute and has been 
				  classified as PUBLIC.
===========================================================================
*/
#pragma once

#pragma warning(push)     // disable for this header only
#pragma warning(disable:4312) 
// DirectShow
#include <Streams.h>
#pragma warning(pop)      // restore original warning level
// STL
#include <string>
#include <queue>

// Forward
class MediaSample;
class RtspSourceFilter;
class MediaSubsession;

class RtspSourceOutputPin : public CSourceStream,
							public IAMPushSource
{
	///this needs to be declared for the extra interface (adds the COM AddRef, etc methods)
	DECLARE_IUNKNOWN; // Need this for the IAMPushSource interface

	friend class RtspSourceFilter;
public:
	/// Constructor
	RtspSourceOutputPin(HRESULT* pHr, RtspSourceFilter* pFilter, MediaSubsession* pMediaSubsession, int nID);

	/// Destructor
	virtual ~RtspSourceOutputPin(void);

	/// We do not currently support quality control
	STDMETHODIMP Notify(IBaseFilter* pSelf, Quality q)
	{
		return E_FAIL;
	}

	/// override this to publicise our interfaces
	STDMETHODIMP NonDelegatingQueryInterface(REFIID riid, __deref_out void **ppv)
	{
		if (riid == IID_IAMPushSource)
			return GetInterface((IAMPushSource *)this, ppv);
		else
			return CSourceStream::NonDelegatingQueryInterface(riid, ppv); 
	}

	/// From IAMPushSource
	STDMETHODIMP GetMaxStreamOffset(REFERENCE_TIME *prtMaxOffset){return E_NOTIMPL;}
	/// From IAMPushSource
	STDMETHODIMP GetPushSourceFlags(ULONG *pFlags)
	{ 
		*pFlags = AM_PUSHSOURCECAPS_PRIVATE_CLOCK;return S_OK;
	}
	/// From IAMPushSource
	STDMETHODIMP GetStreamOffset(REFERENCE_TIME *prtOffset){return E_NOTIMPL;}
	/// From IAMPushSource
	STDMETHODIMP SetMaxStreamOffset(REFERENCE_TIME rtMaxOffset)	{return E_NOTIMPL;}
	/// From IAMPushSource
	STDMETHODIMP SetPushSourceFlags(ULONG Flags)	{return E_NOTIMPL;}
	/// From IAMPushSource
	STDMETHODIMP SetStreamOffset(REFERENCE_TIME rtOffset){return E_NOTIMPL;}
	/// From IAMPushSource
	STDMETHODIMP GetLatency(REFERENCE_TIME *prtLatency)
	{ 
		return E_NOTIMPL;
		//TODO: set different latencies for audio and video
		//Set 450ms Latency 
		*prtLatency = 45000000; return S_OK;
	}

	/// Sets the start time offset of the filter: this offset gets subtracted from the starting time of each media sample
	/// This has to happen since the samples we receive will not be Zero-based. 
	void setOffset(double dOffset);

protected:

	void initialiseMediaType(MediaSubsession* pMediaSubsession, HRESULT* phr);

	///Override CSourceStream methods
	virtual HRESULT GetMediaType(CMediaType* pMediaType);

	/// Decides how big the buffer must be for the media sample
	HRESULT DecideBufferSize(IMemAllocator* pAlloc, ALLOCATOR_PROPERTIES* pRequestProperties);

	/// method which fills frame buffer with media sample data and sets start times
	HRESULT FillBuffer(IMediaSample* pSample);

	/// Buffer processing loop
	virtual HRESULT DoBufferProcessingLoop(void);    // the loop executed whilst running
	
private:
	/// Source filter
	RtspSourceFilter* m_pFilter;
	/// Protects our internal state
	CCritSec m_cSharedState;

	/// Media type of this output pin
	CMediaType* m_pMediaType;

	// Audio parameters
	int m_nBitsPerSample;
	int m_nChannels;
	int m_nBytesPerSecond;
	int m_nSamplesPerSecond;

	// ID
	int m_nID;

	/// Starting time offset: the time that the first sample arrived. This is used to offset further start times to 0 
	bool m_bOffsetSet;
	/// Whether the pin has received a synchronisation sample yet
	bool m_bHasBeenSynced;

	/// Stores the global base starting time
	double m_dGlobalStartTime;
	/// Local offset: this is used when one pin's starting time has been synchronised yet the other pins hasn't
	/// As a result this pin should still use it's original offset until it too is synchronised
	double m_dLocalStartTime;
};
