/** @file
===========================================================================
MODULE				: CScaleFilter

DESCRIPTIONS		: Scale Filter for RGB24 Media

REVISION HISTORY	: Version 1.1
					  1) 15/02/2008 RG: Fixed possible NULL pointer in SetParameter that can occur before input pin is connected
					  2) 15/02/2008 RG: Renamed variable names and changed SettingsInterface signatures to be more intuitive


COPYRIGHT			: (c)CSIR, Meraka Institute 2007 all rights reserved

RESTRICTIONS		: The information/data/code contained within this file is 
						the property of CSIR, Meraka Institute and has been 
						classified as PUBLIC.
===========================================================================
*/
#include "stdafx.h"

// Meraka includes
#include "ScaleFilter.h"

#include <DirectShow/CustomMediaTypes.h>
#include <Image/PicScalerRGB24Impl.h>
#include <Image/PicScalerYUV420PImpl.h>

CScaleFilter::CScaleFilter()
: CCustomBaseFilter(NAME("Meraka RTVC Scale Filter"), 0, CLSID_ScaleFilter),
	m_pScaler(NULL),
	m_nBytesPerPixel(BYTES_PER_PIXEL_RGB24)
{
	//Call the initialize input method to load all acceptable input types for this filter
	InitialiseInputTypes();
	// Default scale parameters
	m_nOutWidth = 176;
	m_nOutHeight = 144;
}

CScaleFilter::~CScaleFilter()
{
	if (m_pScaler)
	{
		delete m_pScaler;
		m_pScaler = NULL;
	}
}

CUnknown * WINAPI CScaleFilter::CreateInstance( LPUNKNOWN pUnk, HRESULT *pHr )
{
	CScaleFilter *pFilter = new CScaleFilter();
	if (pFilter== NULL) 
	{
		*pHr = E_OUTOFMEMORY;
	}
	return pFilter;
}


void CScaleFilter::InitialiseInputTypes()
{
	AddInputType(&MEDIATYPE_Video, &MEDIASUBTYPE_RGB24, &FORMAT_VideoInfo);
	AddInputType(&MEDIATYPE_Video, &MEDIASUBTYPE_YUV420P, &FORMAT_VideoInfo);
}

HRESULT CScaleFilter::SetMediaType( PIN_DIRECTION direction, const CMediaType *pmt )
{
	HRESULT hr = CCustomBaseFilter::SetMediaType(direction, pmt);
	if (direction == PINDIR_INPUT)
	{
		//Set defaults
		if (m_nOutWidth == 0)
		{
			m_nOutWidth = m_nInWidth;
		}
		if (m_nOutHeight == 0)
		{
			m_nOutHeight = m_nInHeight;
		}
		//Determine whether we are connected to a RGB24 or 32 source
		if (pmt->majortype == MEDIATYPE_Video)
		{
			//The scaler might already exist if the filter has been connected previously
			if (m_pScaler)
			{
				delete m_pScaler;
				m_pScaler = NULL;
			}
			if (pmt->subtype == MEDIASUBTYPE_RGB24)
			{
				m_pScaler = new PicScalerRGB24Impl();
				m_nBytesPerPixel = BYTES_PER_PIXEL_RGB24;
			}
			else if (pmt->subtype == MEDIASUBTYPE_YUV420P)
			{
				m_pScaler = new PicScalerYUV420PImpl();
				m_nBytesPerPixel = BYTES_PER_PIXEL_YUV420P;
			}
		}
	}
	return hr;
}

HRESULT CScaleFilter::GetMediaType( int iPosition, CMediaType *pMediaType )
{
	if (iPosition < 0)
	{
		return E_INVALIDARG;
	}
	else if (iPosition == 0)
	{
		// Get the input pin's media type and return this as the output media type - we want to retain
		// all the information about the image
		HRESULT hr = m_pInput->ConnectionMediaType(pMediaType);
		if (FAILED(hr))
		{
			return hr;
		}

		// Get the bitmap info header and adapt the cropped 
		//make sure that it's a video info header
		ASSERT(pMediaType->formattype == FORMAT_VideoInfo);
		VIDEOINFOHEADER *pVih = (VIDEOINFOHEADER*)pMediaType->pbFormat;
		//Now we need to calculate the size of the output image
		BITMAPINFOHEADER* pBi = &(pVih->bmiHeader);
		
		// Set height
		pBi->biHeight = m_nOutHeight;
		ASSERT(pBi->biHeight > 0);

		// Set width
		pBi->biWidth = m_nOutWidth;
		ASSERT(pBi->biWidth > 0);
		// Set size
		pBi->biSizeImage = pBi->biWidth * pBi->biHeight * BYTES_PER_PIXEL_RGB24;
		pMediaType->lSampleSize = pBi->biSizeImage;

		return S_OK;
	}
	return VFW_S_NO_MORE_ITEMS;
}

HRESULT CScaleFilter::DecideBufferSize( IMemAllocator *pAlloc, ALLOCATOR_PROPERTIES *pProp )
{
	// Leaving this var so that we can cater for RGB32 at a later stage
	pProp->cbBuffer = m_nOutPixels * m_nBytesPerPixel;
	if (m_nBytesPerPixel == BYTES_PER_PIXEL_YUV420P)
	{
		//Adjust the buffer requirements for our custom format
		pProp->cbBuffer *= sizeof(yuvType);
	}

	if (pProp->cbAlign == 0)
	{
		pProp->cbAlign = 1;
	}
	if (pProp->cBuffers == 0)
	{
		pProp->cBuffers = 1;
	}
	
	// Set allocator properties.
	ALLOCATOR_PROPERTIES Actual;
	HRESULT hr = pAlloc->SetProperties(pProp, &Actual);
	if (FAILED(hr)) 
	{
		return hr;
	}
	// Even when it succeeds, check the actual result.
	if (pProp->cbBuffer > Actual.cbBuffer) 
	{
		return E_FAIL;
	}
	return S_OK;
}

HRESULT CScaleFilter::CheckTransform( const CMediaType *mtIn, const CMediaType *mtOut )
{
	//Make sure the input and output types are related
	if (mtOut->majortype != MEDIATYPE_Video)
	{
		return VFW_E_TYPE_NOT_ACCEPTED;
	}
	//Subtypes
	if (mtIn->subtype == MEDIASUBTYPE_RGB24)
	{
		if (mtOut->subtype != MEDIASUBTYPE_RGB24)
		{
			return VFW_E_TYPE_NOT_ACCEPTED;
		}
	}
	if (mtIn->subtype == MEDIASUBTYPE_YUV420P)
	{
		if (mtOut->subtype != MEDIASUBTYPE_YUV420P)
		{
			return VFW_E_TYPE_NOT_ACCEPTED;
		}
	}
	// Video format
	if (mtOut->formattype != FORMAT_VideoInfo)
	{
		return VFW_E_TYPE_NOT_ACCEPTED;
	}
	return S_OK;
}


STDMETHODIMP CScaleFilter::SetParameter( const char* type, const char* value )
{
	// For now, one cannot set any parameters once the output has been connected -> this will affect the buffersize
	if (m_pOutput)
	{
		if (m_pOutput->IsConnected())
		{
			return E_FAIL;
		}
	}

	// TODO: get rid of enum or get rid of string!!!!
	if (SUCCEEDED(CSettingsInterface::SetParameter(type, value)))
	{
		m_nOutPixels = m_nOutWidth * m_nOutHeight;
		return S_OK;
	}
	else
	{
		return E_FAIL;
	}
}

DWORD CScaleFilter::ApplyTransform( BYTE* pBufferIn, BYTE* pBufferOut )
{
	int nTotalSize = 0;
	//make sure we were able to initialize our converter
	if (m_pScaler)
	{
		//Call scaling conversion code
		m_pScaler->SetInDimensions(m_nInWidth, m_nInHeight);
		m_pScaler->SetOutDimensions(m_nOutWidth, m_nOutHeight);
		m_pScaler->Scale((void*)pBufferOut, (void*)pBufferIn);
		nTotalSize = m_nOutWidth * m_nOutHeight * m_nBytesPerPixel;
	}
	else
	{
		DbgLog((LOG_TRACE, 0, TEXT("Scaler is not initialised - unable to transform")));
	}
	return nTotalSize;
}


HRESULT CScaleFilter::RecalculateFilterParameters()
{
	// Update the number of out pixels
	m_nOutPixels = m_nOutWidth * m_nOutHeight;
	return S_OK;
}